
/*******************************************************************************

 Copyright (c) 2009, Charles McGarvey
 All rights reserved.
 
 Redistribution   and   use  in  source  and  binary  forms,  with  or  without
 modification, are permitted provided that the following conditions are met:
 
   * Redistributions  of  source  code  must retain the above copyright notice,
     this list of conditions and the following disclaimer.
   * Redistributions  in binary form must reproduce the above copyright notice,
     this  list of conditions and the following disclaimer in the documentation
     and/or other materials provided with the distribution.
 
 THIS  SOFTWARE  IS  PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND  ANY  EXPRESS  OR  IMPLIED  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED.  IN  NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 FOR  ANY  DIRECT,  INDIRECT,  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES  (INCLUDING,  BUT  NOT  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 SERVICES;  LOSS  OF  USE,  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 CAUSED  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 OR  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*******************************************************************************/

#ifndef _CHARACTER_HH_
#define _CHARACTER_HH_

#include <boost/shared_ptr.hpp>

#include <Moof/Aabb.hh>
#include <Moof/Entity.hh>
#include <Moof/Math.hh>
#include <Moof/Octree.hh>
#include <Moof/RigidBody.hh>
#include <Moof/Sphere.hh>

#include "Animation.hh"
#include "Tilemap.hh"



class Character;
typedef boost::shared_ptr<Character> CharacterP;


/**
 * Parent class of animate objects with "personalities."  This basically
 * includes the heroine herself and the bad guys.
 */

class Character : public Mf::RigidBody2, public Mf::OctreeInsertable
{
public:

	Character(const std::string& name);
	virtual ~Character() {}

	virtual void update(Mf::Scalar t, Mf::Scalar dt);
	virtual void draw(Mf::Scalar alpha) const;

	void setZCoord(Mf::Scalar z);

	void addImpulse(Mf::Vector2 impulse);
	void addForce(Mf::Vector2 force);
	void setPosition(Mf::Vector2 position);

	virtual int getOctant(const Mf::Aabb& aabb) const;

	Tilemap		tilemap;
	Animation	animation;

private:

	mutable Mf::Scalar mZCoord;
};


#endif // _CHARACTER_HH_

/** vim: set ts=4 sw=4 tw=80: *************************************************/

